import unittest
from nose.plugins.skip import SkipTest


class TestSimple(unittest.TestCase):
    """Specifically tests when inheriting from unittest.TestCase"""
    def test_simple_fail(self):
        assert 1 == 0

    def test_simple_ok(self):
        assert 1 == 1

    def test_simple_skip(self):
        raise SkipTest()



from unittest import SkipTest
from nosedep import depends


@depends(after='test_dfds_b')
def test_dfds_a():
    pass


def test_dfds_b():
    raise AssertionError("UNGA HAZA")


def test_dfds_c():
    pass


@depends(after='not_existing_test')
def test_dfds_d():
    pass


def test_dfds_e():
    raise SkipTest('skippington')


@depends(after=test_dfds_e)
def test_dfds_f():
    pass



from nosedep import depends


class TestMP(object):
    @depends(priority=2)
    def test_dmp_a(self):
        pass

    @depends(priority=1)
    def test_dmp_b(self):
        pass

    @depends(priority=3)
    def test_dmp_c(self):
        pass

    @depends(priority=5, after='test_dmp_f')
    def test_dmp_d(self):
        pass

    @depends(priority=5, after='test_dmp_d')
    def test_dmp_e(self):
        pass

    @depends(priority=5)
    def test_dmp_f(self):
        pass

    @depends(priority=1, after='test_dmp_a')
    def test_dmp_g(self):
        pass

    @depends(priority=2, after='test_dmp_a')
    def test_dmp_h(self):
        pass

    @depends(priority=3, after='test_dmp_a')
    def test_dmp_i(self):
        pass


# Order should be
# NP: b1, c3
# PG1: f5, a2
# PG2: d5,g1,h2,i3
# PG3: e5
#
# => b, c, a, f, g, h, i, d, e



import unittest
from nosedep import depends


class DisplayNodeList(unittest.TestCase):
    @depends(after='test_02')
    def test_01(self):
        pass

    @depends(after='test_03')
    def test_02(self):
        pass

    @depends(after='test_04')
    def test_03(self):
        pass

    def test_04(self):
        pass



"""Nosetest plugin for test dependencies.

Normally tests should not depend on each other - and it should be avoided
as long as possible. Optimally each test should be able to run in isolation.

However there might be rare cases or special circumstances where one would
want this. For example very slow integration tests where redoing what test
A did just to run test B would simply be too costly. Or temporarily while
testing or debugging. It's also possible that one wants some test to run first
as 'smoke tests' such that the rest can be skipped if those tests fail.

The current implementation allows marking tests with the `@depends` decorator
where it can be declared if the test needs to run before or after some
other specific test(s).

There is also support for skipping tests based on the dependency results,
thus if test B depends on test A and test A fails then B will be skipped
with the reason that A failed.

Nosedep also supports running the necessary dependencies for a single test,
thus if you specify to run only test B and test B depends on A; then A will
run before B to satisfy that dependency.

Note that 'before' dependencies are treated as soft. A soft dependency will only
affect the test ordering, not force inclusion. For example if we have::

    def test_a:
      pass

    @depends(before=test_a)
    def test_b:
      pass

and run all tests they would run in the order b,a. If you specify to run only
either one of them only that test would run. However changing it to::

    @depends(after=test_b)
    def test_a:
      pass

    def test_b:
      pass

would affect the case when you specify to run only test a, since it would have
to run test b first to specify the 'after' dependency since it's a 'hard' dependency.

Finally there is prioritization support. Each test can be given an integer priority
and the tests will run in order from lowest to highest. Dependencies take
precedence so in total the ordering will be:

1. All tests with a priority lower or equal to the default that are not part of any
   dependency chain ordered first by priority then by name.
2. Priority groups in order, while each priority group is internally ordered
   the same as point 1.
3. All tests with priority higher than the default that are not part of any
   dependency chain ordered first by priority then by name.

Default priority if not specified is 50.
"""
import imp
import inspect
import os
import re
from collections import defaultdict
from functools import partial, wraps
from itertools import chain, tee

from nose.loader import TestLoader
from nose.plugins import Plugin
from nose.suite import ContextSuite
try:
    # Older versions of setuptools
    from setuptools.compat import reraise
except ImportError:
    from setuptools.extern.six import reraise
from toposort import toposort

dependencies = defaultdict(set)
soft_dependencies = defaultdict(set)
default_priority = 50
priorities = defaultdict(lambda: default_priority)


def depends(func=None, after=None, before=None, priority=None):
    """Decorator to specify test dependencies

    :param after: The test needs to run after this/these tests. String or list of strings.
    :param before: The test needs to run before this/these tests. String or list of strings.
    """
    if not (func is None or inspect.ismethod(func) or inspect.isfunction(func)):
        raise ValueError("depends decorator can only be used on functions or methods")
    if not (after or before or priority):
        raise ValueError("depends decorator needs at least one argument")

    # This avoids some nesting in the decorator
    # If called without func the decorator was called with optional args
    # so we'll return a function with those args filled in.
    if func is None:
        return partial(depends, after=after, before=before, priority=priority)

    def self_check(a, b):
        if a == b:
            raise ValueError("Test '{}' cannot depend on itself".format(a))

    def handle_dep(conditions, _before=True):
        if conditions:
            if type(conditions) is not list:
                conditions = [conditions]
            for cond in conditions:
                if hasattr(cond, '__call__'):
                    cond = cond.__name__
                self_check(func.__name__, cond)
                if _before:
                    soft_dependencies[cond].add(func.__name__)
                else:
                    dependencies[func.__name__].add(cond)

    handle_dep(before)
    handle_dep(after, False)

    if priority:
        priorities[func.__name__] = priority

    @wraps(func)
    def inner(*args, **kwargs):
        return func(*args, **kwargs)
    return inner


def import_from_uri(uri, absl=True):
    if not absl:
        uri = os.path.normpath(os.path.join(os.path.dirname(__file__), uri))
    path, fname = os.path.split(uri)
    mname, ext = os.path.splitext(fname)
    no_ext = os.path.join(path, mname)
    if os.path.exists(no_ext + '.pyc'):
        try:
            return imp.load_compiled(mname, no_ext + '.pyc')
        except ImportError:
            pass
    if os.path.exists(no_ext + '.py'):
        try:
            return imp.load_source(mname, no_ext + '.py')
        except ImportError:
            pass


def has_class(file_name, class_name):
    mod = import_from_uri(file_name)
    if not mod:
        return False
    return bool(inspect.getmembers(mod, lambda o: inspect.isclass(o) and o.__name__ == class_name))


class DepLoader(TestLoader):
    """Loader that stores what was specified but still loads all tests"""

    # noinspection PyPep8Naming
    def __init__(self, config=None, importer=None, workingDir=None, selector=None):
        super(DepLoader, self).__init__(config, importer, workingDir, selector)
        self.tests = []

    def loadTestsFromName(self, name, module=None, discovered=False):
        """Need to load all tests since we might have dependencies"""
        if name == '.':
            # FIXME: This is current workaround that should be handled better
            #        The problem is that the tests in tests.py that do not set
            #        suitepath will cause a call to this function with name '.'
            #        which in turn cause some tests to be tested again.
            #        Use of '.' here cause calls with the absolute path
            #        which on windows contain : and linux not causing us
            #        to handle this case differently.
            #        For now return an empty suite until we can handle this better.
            return self.suiteClass([])
        else:
            # Would have been nice to use nose.util.split_test_name(name) here
            # but for some reason it cause an recursive loop of test loading
            # Need to investigate further why that happens
            parts = name.split(':' if ':' in name else '.') if not name.endswith('.py') else [name]
        if len(parts) == 2 and parts[1]:
            if not has_class(parts[0], parts[1]):
                self.tests.append(parts[-1].split('.')[-1])
        return super(DepLoader, self).loadTestsFromName(parts[0], module, discovered)


def merge_dicts(d1, d2):
    d3 = defaultdict(set)
    for k, v in chain(iter(d1.items()), iter(d2.items())):
        d3[k] |= v
    return d3


def split_on_condition(seq, condition):
    """Split a sequence into two iterables without looping twice"""
    l1, l2 = tee((condition(item), item) for item in seq)
    return (i for p, i in l1 if p), (i for p, i in l2 if not p)


def lo_prio(x):
    return priorities[x] <= default_priority


class NoseDep(Plugin):
    """Allow specifying test dependencies with the depends decorator."""
    name = "nosedep"
    score = 100

    def __init__(self):
        super(NoseDep, self).__init__()
        self.loader = None
        self.ok_results = set()
        self.results = None

    def options(self, parser, env):
        super(self.__class__, self).options(parser, env)

    def configure(self, options, conf):
        super(self.__class__, self).configure(options, conf)

    def prepareTestLoader(self, loader):
        self.loader = DepLoader(loader.config, loader.importer, loader.workingDir, loader.selector)
        return self.loader

    @staticmethod
    def calculate_dependencies():
        """Calculate test dependencies
        First do a topological sorting based on the dependencies.
        Then sort the different dependency groups based on priorities.
        """
        order = []
        for g in toposort(merge_dicts(dependencies, soft_dependencies)):
            for t in sorted(g, key=lambda x: (priorities[x], x)):
                order.append(t)
        return order

    def orderTests(self, all_tests, test):
        """Determine test ordering based on the dependency graph"""
        order = self.calculate_dependencies()
        ordered_all_tests = sorted(list(all_tests.keys()), key=lambda x: (priorities[x], x))
        conds = [lambda t: True, lambda t: t in all_tests]
        if self.loader.tests:  # If specific tests were mentioned on the command line
            def mark_deps(t):
                if t in dependencies:
                    for dep in dependencies[t]:
                        setattr(all_tests[dep], 'nosedep_run', True)
                        mark_deps(dep)

            for t in self.loader.tests:
                if t in all_tests:
                    setattr(all_tests[t], 'nosedep_run', True)
                    mark_deps(t)

            conds[0] = conds[1] = lambda t: t in all_tests and getattr(all_tests[t], 'nosedep_run', False)

        no_deps = (t for t in ordered_all_tests if t not in order and conds[0](t))
        deps = (t for t in order if conds[1](t))
        no_deps_l, no_deps_h = split_on_condition(no_deps, lo_prio)
        test._tests = (all_tests[t] for t in chain(no_deps_l, deps, no_deps_h))
        return test

    def prepare_suite(self, suite):
        """Prepare suite and determine test ordering"""
        all_tests = {}
        for s in suite:
            m = re.match('(\w+)\s+\(.+\)', str(s))
            if m:
                name = m.group(1)
            else:
                name = str(s).split('.')[-1]
            all_tests[name] = s

        return self.orderTests(all_tests, suite)

    def prepareTest(self, test):
        """Prepare and determine test ordering"""
        all_tests = {}

        # When passing a directory to nose we have an extra
        # top level that we need to enter.
        all_tests = self.prepare_tests_on_levels(test, all_tests)
        return self.orderTests(all_tests, test)

    def prepare_tests_on_levels(self, test, all_tests):
        """Find test level of ContextSuite object

        prepare_tests_on_levels() is a recursive function use to find the parent directory of a test or group of tests.
        An attempt is made to iterate through the presented level of a ContextSuite class object. If the iteration
        is successful, the iterated level of ContextSuite is passed to prepare_tests_on_levels() for additional
        processing. If a TypeError is generated, the exception fails back to the previous prepare_tests_on_levels()
        call. The TypeError exception processes this ContextSuite level as the parent level of tests.
        """
        for t in test:
            try:
                self.prepare_tests_on_levels(t, all_tests)
            except TypeError:
                if isinstance(test, ContextSuite):  # MethodTestCase
                    all_tests[test.context.__name__] = self.prepare_suite(test)
                    setattr(all_tests[test.context.__name__], 'nosedep_run', True)
                else:  # FunctionTestCase
                    all_tests[test.test.test.__name__] = test
                break
            except AttributeError:
                # This exception is confusing - reraise the original one
                reraise(test.test.exc_class, test.test.exc_val, test.test.tb)
        return all_tests

    def dependency_failed(self, test):
        """Returns an error string if any of the dependencies failed"""
        for d in (self.test_name(i) for i in dependencies[test]):
            if d in (self.test_name(str(t[0])) for t in self.results.failures):
                return "Required test '{}' FAILED".format(d)
            if d in (self.test_name(str(t[0])) for t in self.results.errors):
                return "Required test '{}' ERRORED".format(d)
            if d in (self.test_name(str(t[0])) for t in self.results.skipped):
                return "Required test '{}' SKIPPED".format(d)
        return None

    def dependency_ran(self, test):
        """Returns an error string if any of the dependencies did not run"""
        for d in (self.test_name(i) for i in dependencies[test]):
            if d not in self.ok_results:
                return "Required test '{}' did not run (does it exist?)".format(d)
        return None

    def beforeTest(self, test):
        """Skip or Error the test if the dependencies are not fulfilled"""
        tn = self.test_name(test)
        res = self.dependency_failed(tn)
        test.test._originalName = test.test._testMethodName
        if res:
            test.test.skipTestNoseDep = partial(test.test.skipTest, res)
            test.test._testMethodName = 'skipTestNoseDep'
            return
        res = self.dependency_ran(tn)
        if res:
            def error_test():
                raise Exception(res)
            test.test.errTestNoseDep = error_test
            test.test._testMethodName = 'errTestNoseDep'

    # noinspection PyMethodMayBeStatic
    def testName(self, test):
        """Implements the plugin interface

        A test that inherits from unittest.TestCase implements __str__ differently
        so we need to work around that by storing the original test name and
        reinsert that here for skips and errors.
        """
        name = str(test.test)
        for prefix in ["skipTestNoseDep", "errTestNoseDep"]:
            if name.startswith(prefix):
                # noinspection PyProtectedMember
                return name.replace(prefix, test.test._originalName)
        return name

    @staticmethod
    def test_name(test):
        # Internally we are currently only using the method/function names
        # could be that we might want to use the full qualified name in the future

        # And if we inherit from unittest.TestCase we need to strip some extra info
        m = re.match('(\w+)\s+\(.+\)', str(test))
        test_name = m.group(1) if m else str(test)

        return test_name.split('.')[-1]

    def addSuccess(self, test):
        """The result object does not store successful results, so we have to do it"""
        self.ok_results.add(self.test_name(test))

    def prepareTestResult(self, result):
        """Store the result object so we can inspect it in beforeTest"""
        self.results = result



import unittest

from nosedep import depends


class TestSimpleDecorated(unittest.TestCase):
    """Specifically tests when inheriting from unittest.TestCase"""
    @depends(after='test_simple_decorated_ok')
    def test_simple_decorated_fail(self):
        assert 1 == 0

    def test_simple_decorated_ok(self):
        assert 1 == 1

    @depends(after='test_simple_decorated_fail')
    def test_simple_decorated_skip(self):
        assert 1 == 1



from nosedep import depends
from nose.tools import assert_true, assert_false


class TestNoseDecoratedParent(object):
    did_setup_class = False
    did_teardown_class = False

    @classmethod
    def reset(cls):
        cls.did_setup_class = False
        cls.did_teardown_class = False

    def __init__(self):
        self.did_setup = False
        self.did_teardown = False

    @classmethod
    def setup_class(cls):
        cls.did_setup_class = True
        # Would be better to reset this externally
        cls.did_teardown_class = False

    @classmethod
    def teardown_class(cls):
        cls.did_teardown_class = True

    def setup(self):
        self.did_setup = True

    def teardown(self):
        self.did_teardown = True


class TestNoseDecoratedChild(TestNoseDecoratedParent):
    def test_cd_a(self):
        assert_true(self.did_setup)
        assert_true(self.did_setup_class)
        assert_false(self.did_teardown)
        assert_false(self.did_teardown_class)

    def test_cd_b(self):
        pass

    @depends(before='test_cd_b')
    def test_cd_c(self):
        pass



import os
import unittest

from nose.plugins import PluginTester
from nose.plugins.skip import Skip
from nose.plugins.xunit import Xunit
from nose.tools import assert_in, eq_

try:
    from nose.tools import assert_raises_regex
except ImportError:
    # For python 2.7
    from nose.tools import assert_raises_regexp as assert_raises_regex
from nosedep import NoseDep, depends


class NoseDepPluginTester(PluginTester, unittest.TestCase):
    activate = '--with-nosedep'
    args = ['-v']
    plugins = [NoseDep(), Skip()]

    # This is a bit odd. If using the absolute path on Windows
    # it tries to import module 'C' due to the ':' , if only relative path
    # it splits on the '.' before the file extension.
    # So I only got it to work with a relative path appended with a ':'.
    suitepath = None

    def makeSuite(self):
        raise Exception("Should not be used currently")

    def check(self, expect):
        results = len(expect)
        for line in self.output:
            if expect:
                self.assertEqual(expect.pop(0), line.strip())
        # Verify that we ran the expected number of tests
        assert_in("Ran {} test{} in".format(results, 's' if results > 1 else ''),
                  str(self.output))


class TestUndecoratedFunctional(NoseDepPluginTester):
    suitepath = "test_scripts/undecorated_functional_tests.py:"

    def runTest(self):
        self.check(['test_scripts.undecorated_functional_tests.test_x ... ok',
                    'test_scripts.undecorated_functional_tests.test_y ... ERROR'])


class TestUndecoratedFunctionalXunit(NoseDepPluginTester):
    # Initially there was a bug causing Xunit to fail in
    # combination with nosedep. This test verifies that it works.
    args = ['-v', '--with-xunit']
    plugins = [NoseDep(), Xunit()]
    suitepath = "test_scripts/undecorated_functional_tests.py:"

    def setUp(self):
        if os.path.isfile('nosetests.xml'):
            os.remove('nosetests.xml')
        super(TestUndecoratedFunctionalXunit, self).setUp()

    def tearDown(self):
        with open('nosetests.xml') as xml:
            assert_in('tests="2"', xml.read())
        super(TestUndecoratedFunctionalXunit, self).tearDown()

    def runTest(self):
        self.check(['test_scripts.undecorated_functional_tests.test_x ... ok',
                    'test_scripts.undecorated_functional_tests.test_y ... ERROR'])


class TestDecoratedClass(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_class.py"

    def setUp(self):
        try:
            super(TestDecoratedClass, self).setUp()
        except ValueError as e:
            eq_(str(e), "depends decorator can only be used on functions or methods")


class TestSimple(NoseDepPluginTester):
    suitepath = "test_scripts/simple.py"

    def runTest(self):
        self.check(['test_simple_fail (test_scripts.simple.TestSimple) ... FAIL',
                    'test_simple_ok (test_scripts.simple.TestSimple) ... ok',
                    'test_simple_skip (test_scripts.simple.TestSimple) ... SKIP'])


class TestSimpleDecorated(NoseDepPluginTester):
    suitepath = "test_scripts/simple_decorated.py"

    def runTest(self):
        self.check(['test_simple_decorated_ok (test_scripts.simple_decorated.TestSimpleDecorated) ... ok',
                    'test_simple_decorated_fail (test_scripts.simple_decorated.TestSimpleDecorated) ... FAIL',
                    'test_simple_decorated_skip (test_scripts.simple_decorated.TestSimpleDecorated) ... SKIP:'
                    ' Required test \'test_simple_decorated_fail\' FAILED'])


class TestSimpleColon(NoseDepPluginTester):
    suitepath = "test_scripts/simple.py:"

    def runTest(self):
        self.check(['test_simple_fail (test_scripts.simple.TestSimple) ... FAIL',
                    'test_simple_ok (test_scripts.simple.TestSimple) ... ok',
                    'test_simple_skip (test_scripts.simple.TestSimple) ... SKIP'])


class TestSimpleClass(NoseDepPluginTester):
    suitepath = "test_scripts/simple.py:TestSimple"

    def runTest(self):
        self.check(['test_simple_fail (test_scripts.simple.TestSimple) ... FAIL',
                    'test_simple_ok (test_scripts.simple.TestSimple) ... ok',
                    'test_simple_skip (test_scripts.simple.TestSimple) ... SKIP'])


class TestSimpleClassTestOK(NoseDepPluginTester):
    suitepath = "test_scripts/simple.py:TestSimple.test_simple_ok"

    def runTest(self):
        self.check(['test_simple_ok (test_scripts.simple.TestSimple) ... ok'])


class TestSimpleClassTestSKIP(NoseDepPluginTester):
    suitepath = "test_scripts/simple.py:TestSimple.test_simple_skip"

    def runTest(self):
        self.check(['test_simple_skip (test_scripts.simple.TestSimple) ... SKIP'])


class TestSimpleClassTestFAIL(NoseDepPluginTester):
    suitepath = "test_scripts/simple.py:TestSimple.test_simple_fail"

    def runTest(self):
        self.check(['test_simple_fail (test_scripts.simple.TestSimple) ... FAIL'])


class TestDecoratedFunctionalAll(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_functional_tests.py:"

    def runTest(self):
        self.check(['test_scripts.decorated_functional_tests.test_dft_f ... ok',
                    'test_scripts.decorated_functional_tests.test_dft_b ... ok',
                    'test_scripts.decorated_functional_tests.test_dft_e ... ok',
                    'test_scripts.decorated_functional_tests.test_dft_a ... ok',
                    'test_scripts.decorated_functional_tests.test_dft_c ... ok',
                    'test_scripts.decorated_functional_tests.test_dft_d ... ok'])


class TestDecoratedFunctionalDepSkip(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_functional_dep_skip.py:"

    def runTest(self):
        self.check(['test_scripts.decorated_functional_dep_skip.test_dfds_c ... ok',
                    'test_scripts.decorated_functional_dep_skip.test_dfds_b ... FAIL',
                    'test_scripts.decorated_functional_dep_skip.test_dfds_e ... SKIP: skippington',
                    'test_scripts.decorated_functional_dep_skip.test_dfds_a ... SKIP:'
                    ' Required test \'test_dfds_b\' FAILED',
                    'test_scripts.decorated_functional_dep_skip.test_dfds_d ... ERROR',
                    'test_scripts.decorated_functional_dep_skip.test_dfds_f ... SKIP:'
                    ' Required test \'test_dfds_e\' SKIPPED'])


class TestDecoratedFunctionalFunc(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_functional_func.py:"

    def runTest(self):
        self.check(['test_scripts.decorated_functional_func.test_dff_b ... ok',
                    'test_scripts.decorated_functional_func.test_dff_a ... ok',
                    'test_scripts.decorated_functional_func.test_dff_c ... ok'])


class TestDecoratedFunctionalMult(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_functional_mult.py:"

    def runTest(self):
        self.check(['test_scripts.decorated_functional_mult.test_dfm_b ... ok',
                    'test_scripts.decorated_functional_mult.test_dfm_c ... ok',
                    'test_scripts.decorated_functional_mult.test_dfm_a ... ok'])


class TestDecoratedFunctionalSpecificDep(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_functional_tests.py:test_dft_d"

    def runTest(self):
        self.check(['test_scripts.decorated_functional_tests.test_dft_b ... ok',
                    'test_scripts.decorated_functional_tests.test_dft_a ... ok',
                    'test_scripts.decorated_functional_tests.test_dft_d ... ok'])


class TestDecoratedFunctionalSpecificNoDep(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_functional_tests.py:test_dft_f"

    def runTest(self):
        self.check(['test_scripts.decorated_functional_tests.test_dft_f ... ok'])


class TestDecoratedFunctionalSpecificC(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_functional_tests.py:test_dft_c"

    def runTest(self):
        self.check(['test_scripts.decorated_functional_tests.test_dft_b ... ok',
                    'test_scripts.decorated_functional_tests.test_dft_c ... ok'])


class TestUndecoratedMethod(NoseDepPluginTester):
    suitepath = "test_scripts/undecorated_method_tests.py:"

    def runTest(self):
        self.check(['test_scripts.undecorated_method_tests.TestNoseUndecoratedMethod.test_c_a ... ok',
                    'test_scripts.undecorated_method_tests.TestNoseUndecoratedMethod.test_c_b ... ok'])


class TestDecoratedMethod(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_method_tests.py:"

    def runTest(self):
        self.check(['test_scripts.decorated_method_tests.TestNoseDecoratedMethod.test_cd_a ... ok',
                    'test_scripts.decorated_method_tests.TestNoseDecoratedMethod.test_cd_c ... ok',
                    'test_scripts.decorated_method_tests.TestNoseDecoratedMethod.test_cd_b ... ok'])


class TestDecoratedSubclass(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_method_subclass.py:"

    def runTest(self):
        self.check(['test_scripts.decorated_method_subclass.TestNoseDecoratedChild.test_cd_a ... ok',
                    'test_scripts.decorated_method_subclass.TestNoseDecoratedChild.test_cd_c ... ok',
                    'test_scripts.decorated_method_subclass.TestNoseDecoratedChild.test_cd_b ... ok'])


class TestDecoratedSubclassTestCase(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_method_subclass_testcase.py:"

    def runTest(self):
        self.check(
            ['test_cdt_a (test_scripts.decorated_method_subclass_testcase.TestNoseDecoratedTestCaseChild) ... ok',
             'test_cdt_c (test_scripts.decorated_method_subclass_testcase.TestNoseDecoratedTestCaseChild) ... ok',
             'test_cdt_b (test_scripts.decorated_method_subclass_testcase.TestNoseDecoratedTestCaseChild) ... ok'])


class TestDecoratedMethodSpecificNoDep(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_method_tests.py:TestNoseDecoratedMethod.test_cd_a"

    def runTest(self):
        self.check(['test_scripts.decorated_method_tests.TestNoseDecoratedMethod.test_cd_a ... ok'])


class TestDecoratedMethodSpecificDep(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_method_tests.py:TestNoseDecoratedMethod.test_cd_b"

    def runTest(self):
        self.check(['test_scripts.decorated_method_tests.TestNoseDecoratedMethod.test_cd_b ... ok'])


class TestDecoratedFunctionalPriority(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_functional_priority.py:"

    def runTest(self):
        self.check(['test_scripts.decorated_functional_priority.test_dfp_b ... ok',
                    'test_scripts.decorated_functional_priority.test_dfp_a ... ok',
                    'test_scripts.decorated_functional_priority.test_dfp_f ... ok',
                    'test_scripts.decorated_functional_priority.test_dfp_g ... ok',
                    'test_scripts.decorated_functional_priority.test_dfp_h ... ok',
                    'test_scripts.decorated_functional_priority.test_dfp_i ... ok',
                    'test_scripts.decorated_functional_priority.test_dfp_d ... ok',
                    'test_scripts.decorated_functional_priority.test_dfp_e ... ok',
                    'test_scripts.decorated_functional_priority.test_dfp_c ... ok'])


class TestDecoratedMethodPriority(NoseDepPluginTester):
    suitepath = "test_scripts/decorated_method_priority.py:"

    def runTest(self):
        self.check(['test_scripts.decorated_method_priority.TestMP.test_dmp_b ... ok',
                    'test_scripts.decorated_method_priority.TestMP.test_dmp_c ... ok',
                    'test_scripts.decorated_method_priority.TestMP.test_dmp_a ... ok',
                    'test_scripts.decorated_method_priority.TestMP.test_dmp_f ... ok',
                    'test_scripts.decorated_method_priority.TestMP.test_dmp_g ... ok',
                    'test_scripts.decorated_method_priority.TestMP.test_dmp_h ... ok',
                    'test_scripts.decorated_method_priority.TestMP.test_dmp_i ... ok',
                    'test_scripts.decorated_method_priority.TestMP.test_dmp_d ... ok',
                    'test_scripts.decorated_method_priority.TestMP.test_dmp_e ... ok'])


class TestDecoratedDirectory2Levels(NoseDepPluginTester):
    suitepath = "test_scripts/dir_test/dir_test_2:"

    def runTest(self):
        self.check(['test_04 (dir_test_2.dir_tests.DisplayNodeList) ... ok',
                    'test_03 (dir_test_2.dir_tests.DisplayNodeList) ... ok',
                    'test_02 (dir_test_2.dir_tests.DisplayNodeList) ... ok',
                    'test_01 (dir_test_2.dir_tests.DisplayNodeList) ... ok'])


class TestDecoratedDirectoryDirect(NoseDepPluginTester):
    suitepath = "test_scripts/dir_test/dir_test_2/dir_tests.py:"

    def runTest(self):
        self.check(['test_04 (dir_test_2.dir_tests.DisplayNodeList) ... ok',
                    'test_03 (dir_test_2.dir_tests.DisplayNodeList) ... ok',
                    'test_02 (dir_test_2.dir_tests.DisplayNodeList) ... ok',
                    'test_01 (dir_test_2.dir_tests.DisplayNodeList) ... ok'])


class TestDependsNoArgument(NoseDepPluginTester):
    def test_no_args(self):
        pass

    def makeSuite(self):
        with assert_raises_regex(ValueError, r'depends decorator needs at least one argument'):
            class TC(unittest.TestCase):
                @depends()
                def run_test_no_args(self):
                    pass

            return [TC('run_test_no_args')]


class TestDependsSelf(NoseDepPluginTester):
    def test_self_dep(self):
        pass

    def makeSuite(self):
        with assert_raises_regex(ValueError, r'Test \'run_test_self_dep\' cannot depend on itself'):
            class TC(unittest.TestCase):
                @depends(after='run_test_self_dep')
                def run_test_self_dep(self):
                    pass

            return [TC('run_test_self_dep')]


if __name__ == '__main__':
    unittest.main()


# To test:
# * Multiple specifiers
# * Test both with file and __main__ specifying prefix
#   In the tested application use of __main__. tests all tests while filename prefix works.

# FIXME: Dependencies are global and not qualified. Would probably be best if
#        it contained unique qualifiers. (now we can't use the same test name in different files)



from nosedep import depends, default_priority


@depends(priority=2)
def test_dfp_a():
    pass


@depends(priority=1)
def test_dfp_b():
    pass


# Setting priority above the default to force this
# to be tested last.
@depends(priority=default_priority+1)
def test_dfp_c():
    pass


@depends(priority=5, after='test_dfp_f')
def test_dfp_d():
    pass


@depends(priority=5, after='test_dfp_d')
def test_dfp_e():
    pass


@depends(priority=5)
def test_dfp_f():
    pass


@depends(priority=1, after='test_dfp_a')
def test_dfp_g():
    pass


@depends(priority=2, after='test_dfp_a')
def test_dfp_h():
    pass


@depends(priority=3, after='test_dfp_a')
def test_dfp_i():
    pass



from nosedep import depends


@depends(after='test_dft_b')
def test_dft_a():
    pass


def test_dft_b():
    pass


@depends(after='test_dft_b')
def test_dft_c():
    pass


@depends(after='test_dft_a')
def test_dft_d():
    pass


@depends(before='test_dft_c')
def test_dft_e():
    pass


def test_dft_f():
    pass



from setuptools import setup

setup_args = dict(
    name='nosedep',
    version='0.6',
    url='https://github.com/Zitrax/nose-dep',
    maintainer='Daniel Bengtsson',
    maintainer_email='daniel@bengtssons.info',
    description='Nose test dependency support',
    long_description=open('README.md').read(),
    license='MIT',
    py_modules=['nosedep'],
    zip_safe=False,
    entry_points={
        'nose.plugins.0.10': [
            'nosedep = nosedep:NoseDep'
            ]
        },
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ]
    )

if __name__ == '__main__':
    with open('requirements.txt') as f:
        required = f.read().splitlines()

    setup(install_requires=required,
          **setup_args)



import unittest

from nosedep import depends


def test_decorated_class_dep():
    assert 1


@depends(after=test_decorated_class_dep)
class TestSimpleDecoratedClass(unittest.TestCase):
    def test_decorated_class_internal(self):
        assert 1
